-- puzzle using sprite batch

local gr, mg, ky, lm	= love.graphics, love.image, love.keyboard, love.mouse
local pi, sin, cos		= math.pi, math.sin, math.cos
--[[
Key variables used:
	mouse & sel (as in selected) are in screen coordinates
	mx,my & sx,sy are above in grid coordinates
	find_grid_location( m ) converts screen to grid coordinates (mouse => mx,my)
	qd[y][x] & sb are a quad and the spritebatch containing the quads
	cell[y][x] contains grid cell information - (rotation, quad ID)
	rot[num] contains angle and needed coordinate offsets (only 90 degree steps)
	size determines grid square size
	steps.x & steps.y are calculated from image width & height using the size
]]

function love.load()
	screen_settings()
	puzzle_settings()
	end

function love.update( dt )
	mouse  = { lm.getX(), lm.getY() }
	mx, my	= find_grid_location( mouse )		-- mouse currently over qd(my,mx)
	sx, sy	= find_grid_location( sel )			-- selected quad is qd(sy,sx)
	if lm.isDown( 'l' ) then sel = mouse end
	if lm.isDown( 'r' ) then swap_quads() end	-- swap mouse & selected quads
	end

function swap_quads()	-- interchange selected quad with one under mouse position
	if mx <= steps.x and my <= steps.y then	-- if over image
		qd[sy][sx],		qd[my][mx]		= qd[my][mx],	qd[sy][sx]
		cell[sy][sx],	cell[my][mx]	= cell[my][mx],	cell[sy][sx]
		reload_quad_to_sb(sy,sx)		-- replace old quads in sb with new
		reload_quad_to_sb(my,mx)
		sel	= mouse
		end
	end

function reload_quad_to_sb(y,x)
	sb:setq(
		cell[y][x][2], qd[y][x],					-- quad ID & quad
		size * (x-1), size * (y-1),					-- screen position
		rot[cell[y][x][1]][1],						-- quad rotation
		1,1,										-- quad scaling factor
		rot[cell[y][x][1]][2],rot[cell[y][x][1]][3] -- quad offsets
		)
	end

function love.draw()
	gr.draw( sb )
	gr.print( 'fps: '..love.timer.getFPS( ), width - 95, height - 20 )
	if show.grid then grid() end
	if show.text then add_text() end
	highlight_selected_cells()
	if show.help then								-- overlay image on cells
		gr.setColor( 255, 255, 255, 155 )
		gr.draw( img )
		gr.setColor( 255, 255, 255, 255 )
		end
	end

function love.keypressed( key )
	if key == 'escape'	then love.event.push( 'quit' )	end
	if key == 'right'	then		-- left & right arrow keys rotate quad
		cell[sy][sx][1] = cell[sy][sx][1] < #rot and cell[sy][sx][1] + 1 or 1
		reload_quad_to_sb(sy,sx) end
	if key == 'left'	then
		cell[sy][sx][1] = cell[sy][sx][1] > 1 and cell[sy][sx][1] - 1 or #rot
		reload_quad_to_sb(sy,sx) end
	if key == 'g'		then show.grid	= not show.grid	end
	if key == 'h'		then show.help	= not show.help	end
	if key == 'i'		then show.text	= not show.text	end
	end

function screen_settings()
	img 				= gr.newImage( 'Love.jpg')
	width, height		= img:getWidth(), gr.getHeight()
	show				= {
							grid = true,
							help = false,
							text = true
							}
	text = {
		"left click to select cell",
		"right click to swap cells",
		"< & > to rotate selected cell",
		"<g> to turn off grid",
		"<h> to cheat",
		"<i> to turn off this text",
		"<esc> to exit"
		}
	end

function puzzle_settings()
	gr.setCaption( "Puzzle" )
	size		= 100		-- determines number of squares to be created
	sel			= { 0, 0 }	-- dummy grid selection
	create_quads()
	shuffle_cells()
	randomize_quad_orientations()
	create_sb()
	gr.setFont( gr.newFont( 16 ) )
	end

function create_quads()
	steps	= { x = width / size,	y = height / size }	-- insures square quads
	rot		= {
				{ 0,	0,		0 	},
				{ pi/2,	0,		size},
				{ pi,	size,	size},
				{3*pi/2,size,	0	}
				}
	qd, cell, r, sb = {}, {}, 1, 1		-- r is default rotation (0) via rot[1][1]
	for j = 1, steps.y do
		qd[j], cell[j] = {}, {}
		for i = 1, steps.x do
			qd[j][i] = gr.newQuad( size * (i-1), size * (j-1), size, size, width, height )
			cell[j][i] = { r, sb }	-- cell retains rotation & quad ID info
			end
		end
	end

function create_sb()
	sb=gr.newSpriteBatch( img, steps.x*steps.y)
	sb:clear()
	for j = 1, steps.y do
		for i = 1, steps.x do
			cell[j][i][2] = sb:addq(
					qd[j][i],						-- quad to add
					size * (i-1), size * (j-1),		-- screen position
					rot[cell[j][i][1]][1],			-- quad rotation (effects offsets)
					1,1,							-- quad size
					rot[cell[j][i][1]][2],rot[cell[j][i][1]][3] -- quad offsets
					)
			end
		end
	end

function highlight_selected_cells()
	gr.setColor( 255, 0, 0, 255 )
	draw_box( find_grid_location( mouse ) )
	if sel then
		gr.setColor( 0, 255, 0, 255 )
		draw_box( find_grid_location( sel ) )
		end
	gr.setColor( 255, 255, 255, 255 )
	end

function find_grid_location( m )	-- convert screen coordinates to grid location
	return 1 + math.floor( m[1] / size ), 1 + math.floor( m[2] / size )
	end

function grid()
	gr.setColor( 255, 255, 255, 55 )
	for y = -1, steps.y  do
		for x = -1, steps.x  do
			draw_box( x, y )
			end
		end
	gr.setColor( 255, 255, 255, 255 )
	end

function draw_box( x, y )
	gr.rectangle( 'line', (x-1) * size, (y-1) * size, size, size )
	end

function randomize_quad_orientations()
	for j = 1, steps.y do
		for i = 1, steps.x do
			cell[j][i][1] = math.random( 1, #rot )
			end
		end
	end

function add_text()
	gr.setColor( 255, 255, 0, 255 )
	for i = 1, #text do
		gr.print( text[ i ], 445, 15 * i )
		end
	end

function shuffle_cells()
	for j = 1, steps.y do shuffle_row(j)	end
	for i = 1, steps.x do shuffle_column(i)	end
	end

function shuffle_row(r)
	local n = steps.x
	while n >= 2 do
		local k = math.random(n)
		qd[r][n], qd[r][k] = qd[r][k], qd[r][n]
		n = n - 1
		end
	end

function shuffle_column(c)
	local n = steps.y
	while n >= 2 do
		local k = math.random(n)
		qd[n][c], qd[k][c] = qd[k][c], qd[n][c]
		n = n - 1
		end
	end
